@@ -51,6 +51,7 @@ from brian2.input.spikegeneratorgroup import *
 from brian2.synapses.synapses import StateUpdater as SynapsesStateUpdater
 from brian2.utils.logger import get_logger, std_silent
 from brian2.devices.cpp_standalone.codeobject import CPPStandaloneCodeObject
+from brian2.devices.cpp_standalone.device import CPPWriter
 from brian2 import prefs
 from .codeobject import GeNNCodeObject, GeNNUserCodeObject
 from .genn_generator import get_var_ndim, GeNNCodeGenerator
@@ -224,6 +225,7 @@ class spikegeneratorModel(object):
     '''
     def __init__(self):
         self.name = ''
+        self.codeobject_name = ''
         self.N = 0
 
 
@@ -261,6 +263,7 @@ class spikeMonitorModel(object):
     '''
     def __init__(self):
         self.name = ''
+        self.codeobject_name = ''
         self.neuronGroup = ''
         self.notSpikeGeneratorGroup = True
 
@@ -271,6 +274,7 @@ class rateMonitorModel(object):
     '''
     def __init__(self):
         self.name = ''
+        self.codeobject_name = ''
         self.neuronGroup = ''
         self.notSpikeGeneratorGroup = True
 
@@ -281,8 +285,10 @@ class stateMonitorModel(object):
     '''
     def __init__(self):
         self.name = ''
+        self.codeobject_name = ''
         self.order = 0
         self.monitored = ''
+        self.src = None
         self.isSynaptic = False
         self.variables = []
         self.srcN = 0
@@ -291,32 +297,6 @@ class stateMonitorModel(object):
         self.connectivity = ''
 
 
-class CPPWriter(object):
-    '''
-    Class that provides the method for writing C++ files from a string of code.
-    '''
-    def __init__(self, project_dir):
-        self.project_dir = project_dir
-        self.source_files = []
-        self.header_files = []
-
-    def write(self, filename, contents):
-        logger.diagnostic('Writing file %s:\n%s' % (filename, contents))
-        if filename.lower().endswith('.cpp'):
-            self.source_files.append(filename)
-        elif filename.lower().endswith('.h'):
-            self.header_files.append(filename)
-        elif filename.endswith('.*'):
-            self.write(filename[:-1] + 'cpp', contents.cpp_file)
-            self.write(filename[:-1] + 'h', contents.h_file)
-            return
-        fullfilename = os.path.join(self.project_dir, filename)
-        if os.path.exists(fullfilename):
-            if open(fullfilename, 'r').read() == contents:
-                return
-        open(fullfilename, 'w').write(contents)
-
-
 # ------------------------------------------------------------------------------
 # Start of GeNNDevice
 # ------------------------------------------------------------------------------
@@ -350,9 +330,9 @@ class GeNNDevice(CPPStandaloneDevice):
         self.report_func = ''
         self.src_counts= dict()
         self.trg_counts= dict()
-        #: List of all source and header files (to be included in runner)
-        self.source_files = []
-        self.header_files = []
+        #: Set of all source and header files (to be included in runner)
+        self.source_files = set()
+        self.header_files = set()
 
         self.connectivityDict = dict()
         self.groupDict = dict()
@@ -876,7 +856,7 @@ class GeNNDevice(CPPStandaloneDevice):
         # Create an empty network.h file, this allows us to use Brian2's
         # objects.cpp template unchanged
         writer.write('network.*', GeNNUserCodeObject.templater.network(None, None))
-        self.header_files.append('network.h')
+        self.header_files.add('network.h')
 
         self.generate_objects_source(arange_arrays, self.net,
                                      static_array_specs,
@@ -967,11 +947,11 @@ class GeNNDevice(CPPStandaloneDevice):
                     code = '#include "objects.h"\n' + code
 
                     writer.write('code_objects/' + codeobj.name + '.cpp', code)
-                    self.source_files.append(
+                    self.source_files.add(
                         'code_objects/' + codeobj.name + '.cpp')
                     writer.write('code_objects/' + codeobj.name + '.h',
                                  codeobj.code.h_file)
-                    self.header_files.append(
+                    self.header_files.add(
                         'code_objects/' + codeobj.name + '.h')
 
     def generate_max_row_length_code_objects(self, writer):
@@ -1371,6 +1351,7 @@ class GeNNDevice(CPPStandaloneDevice):
         for obj in spikegenerator_groups:
             spikegenerator_model = spikegeneratorModel()
             spikegenerator_model.name = obj.name
+            spikegenerator_model.codeobject_name = obj.codeobj.name
             spikegenerator_model.N = obj.N
             self.spikegenerator_models.append(spikegenerator_model)
 
@@ -1437,7 +1418,7 @@ class GeNNDevice(CPPStandaloneDevice):
                     self.fix_synapses_code(synapse_model, pathway, codeobj,
                                            code)
 
-            if obj.state_updater != None:
+            if obj.state_updater is not None:
                 codeobj = obj.state_updater.codeobj
                 code = codeobj.code.cpp_file
                 self.collect_synapses_variables(synapse_model, 'dynamics',
@@ -1446,23 +1427,23 @@ class GeNNDevice(CPPStandaloneDevice):
                                        code)
 
             synapse_model.summed_variables = [ s for s in objects if s.startswith(obj.name+'_summed_variable')]
-            if (len(synapse_model.summed_variables) > 0 and hasattr(obj, '_genn_post_write_var')):
+            if len(synapse_model.summed_variables) > 0 and hasattr(obj, '_genn_post_write_var'):
                 raise NotImplementedError("brian2genn only supports a "
                                           "either a single synaptic output variable "
                                           "or a single summed variable per Synapses group.")
-            if (len(synapse_model.summed_variables) > 0 and isinstance(obj.target,Subgroup)):
+            if len(synapse_model.summed_variables) > 0 and isinstance(obj.target,Subgroup):
                 raise NotImplementedError("brian2genn does not support summed variables "
                                           "when the target is a Subgroup.")
-            if (len(synapse_model.summed_variables) > 1):
+            if len(synapse_model.summed_variables) > 1:
                  raise NotImplementedError("brian2genn only supports a "
                                           "single summed variable per Synapses group.")
-            if (hasattr(obj, '_genn_post_write_var')):
+            if hasattr(obj, '_genn_post_write_var'):
                 synapse_model.postSyntoCurrent = '0; $(' + obj._genn_post_write_var.replace(
                     '_post', '') + ') += $(inSyn); $(inSyn)= 0'
             else:
-                if (len(synapse_model.summed_variables) > 0):
+                if len(synapse_model.summed_variables) > 0:
                     summed_variable_updater= objects.get(synapse_model.summed_variables[0], None)
-                    if (obj.target != summed_variable_updater.target):
+                    if obj.target != summed_variable_updater.target:
                         raise NotImplementedError("brian2genn only supports summed "
                                           "variables that target the post-synaptic neuron group of the Synapses the variable is defined in.")
                     synapse_model.postSyntoCurrent = '0; $(' + summed_variable_updater.target_var.name + ') = $(inSyn); $(inSyn)= 0'
@@ -1474,7 +1455,7 @@ class GeNNDevice(CPPStandaloneDevice):
                     addVar = code_generator.translate_expression(addVar)
                     kwds = code_generator.determine_keywords()
                     identifiers = get_identifiers(addVar)
-                    for k,v in iteritems(obj.variables):
+                    for k, v in iteritems(codeobj.variables):
                         if k in ['_spikespace', 't', 'dt'] or k not in identifiers:
                             pass
                         else:
@@ -1549,6 +1530,7 @@ class GeNNDevice(CPPStandaloneDevice):
                     'GeNN does not yet support event monitors for non-spike events.');
             sm = spikeMonitorModel()
             sm.name = obj.name
+            sm.codeobject_name = obj.codeobj.name
             if (hasattr(obj, 'when')):
                 if (not obj.when in ['end', 'thresholds']):
                     # GeNN always records in the end slot but this should
@@ -1568,7 +1550,6 @@ class GeNNDevice(CPPStandaloneDevice):
             if isinstance(src, SpikeGeneratorGroup):
                 sm.notSpikeGeneratorGroup = False
             self.spike_monitor_models.append(sm)
-            self.header_files.append('code_objects/' + sm.name + '_codeobject.h')
 
             # ------------------------------------------------------------------------------
             # Process rate monitors
@@ -1577,6 +1558,7 @@ class GeNNDevice(CPPStandaloneDevice):
         for obj in rate_monitors:
             sm = rateMonitorModel()
             sm.name = obj.name
+            sm.codeobject_name = obj.codeobj.name
             if obj.when != 'end':
                 logger.warn("Rate monitor {!s} has 'when' property '{!s}' which"
                             "is not supported in GeNN, defaulting to"
@@ -1588,18 +1570,18 @@ class GeNNDevice(CPPStandaloneDevice):
             if isinstance(src, SpikeGeneratorGroup):
                 sm.notSpikeGeneratorGroup = False
             self.rate_monitor_models.append(sm)
-            self.header_files.append(
-                'code_objects/' + sm.name + '_codeobject.h')
 
     def process_state_monitors(self, directory, state_monitors, writer):
         for obj in state_monitors:
             sm = stateMonitorModel()
             sm.name = obj.name
+            sm.codeobject_name = obj.codeobj.name
             sm.order = obj.order
             src = obj.source
             if isinstance(src, Subgroup):
                 src = src.source
             sm.monitored = src.name
+            sm.src = src
             sm.when = obj.when
             if sm.when not in ['start', 'end']:
                 logger.warn("State monitor {!s} has 'when' property '{!s}'"
@@ -1632,8 +1614,6 @@ class GeNNDevice(CPPStandaloneDevice):
                     sm.variables.append(varname)
 
             self.state_monitor_models.append(sm)
-            self.header_files.append(
-                'code_objects/' + sm.name + '_codeobject.h')
 
     def generate_model_source(self, writer, main_lines, use_GPU):
         synapses_classes_tmp = CPPStandaloneCodeObject.templater.synapses_classes(None, None)
@@ -1672,14 +1652,14 @@ class GeNNDevice(CPPStandaloneDevice):
         writer.write('magicnetwork_model.cpp', model_tmp)
 
     def generate_main_source(self, writer, main_lines):
-        header_files = self.header_files + prefs['codegen.cpp.headers']
+        header_files = sorted(self.header_files) + prefs['codegen.cpp.headers']
         runner_tmp = GeNNCodeObject.templater.main(None, None,
                                                    code_lines=self.code_lines,
                                                    neuron_models=self.neuron_models,
                                                    synapse_models=self.synapse_models,
                                                    main_lines=main_lines,
                                                    header_files=header_files,
-                                                   source_files=self.source_files,
+                                                   source_files=sorted(self.source_files),
                                                    prefs=prefs,
                                                    )
         writer.write('main.*', runner_tmp)
@@ -1790,8 +1770,8 @@ class GeNNDevice(CPPStandaloneDevice):
             code_objects=the_objects
         )
         writer.write('objects.*', arr_tmp)
-        self.header_files.append('objects.h')
-        self.source_files.append('objects.cpp')
+        self.header_files.add('objects.h')
+        self.source_files.add('objects.cpp')
 
     def copy_source_files(self, writer, directory):
         # Copies brianlib, spikequeue and randomkit
@@ -1805,9 +1785,9 @@ class GeNNDevice(CPPStandaloneDevice):
                                       os.path.join(directory, 'b2glib'))
         for file in b2glib_files:
             if file.lower().endswith('.cpp'):
-                self.source_files.append('b2glib/' + file)
+                self.source_files.add('b2glib/' + file)
             elif file.lower().endswith('.h'):
-                self.header_files.append('b2glib/' + file)
+                self.header_files.add('b2glib/' + file)
 
     def network_run(self, net, duration, report=None, report_period=10 * second,
                     namespace=None, profile=False, level=0, **kwds):
@@ -1824,7 +1804,7 @@ class GeNNDevice(CPPStandaloneDevice):
                 'Only a single run statement is supported for the genn device.')
         self.run_duration = float(duration)
         for obj in net.objects:
-            if obj.clock.name is not 'defaultclock' and not (obj.__class__ == CodeRunner):
+            if obj.clock.name != 'defaultclock' and not (obj.__class__ == CodeRunner):
                 raise NotImplementedError(
                     'Multiple clocks are not supported for the genn device')
 
