@@ -7,7 +7,6 @@
 //--------------------------------------------------------------------------
 
 #include "main.h"
-#include "magicnetwork_model.cpp"
 #include "magicnetwork_model_CODE/definitions.h"
 
 {% for header in header_files %}
@@ -21,36 +20,33 @@
 #include "engine.cpp"
 
 
+
 //--------------------------------------------------------------------------
 /*! \brief This function is the entry point for running the simulation of the MBody1 model network.
 */
 //--------------------------------------------------------------------------
-int which;
-
 int main(int argc, char *argv[])
 {
-  if (argc != 4)
+  if (argc != 3)
   {
-    fprintf(stderr, "usage: main <basename> <time (s)> <CPU=0, GPU=1> \n");
+    fprintf(stderr, "usage: main <basename> <time (s)> \n");
     return 1;
   }
   double totalTime= atof(argv[2]);
-  which= atoi(argv[3]);
-  string OutDir = toString(argv[1]) +"_output";
-  string cmd= toString("mkdir ") +OutDir;
+  string OutDir = std::string(argv[1]) +"_output";
+  string cmd= std::string("mkdir ") +OutDir;
   system(cmd.c_str());
   string name;
-  name= OutDir+ "/"+ toString(argv[1]) + toString(".time");
+  name= OutDir+ "/"+ argv[1] + ".time";
   FILE *timef= fopen(name.c_str(),"a");
 
-  timer.startTimer();
   fprintf(stderr, "# DT %f \n", DT);
   fprintf(stderr, "# totalTime %f \n", totalTime);
 
   {{'\n'.join(code_lines['before_start'])|autoindent}}
 
   //-----------------------------------------------------------------
-  // build the neuronal circuitery
+  // build the neuronal circuitery (calls initialize and allocateMem)
   engine eng;
 
   //-----------------------------------------------------------------
@@ -74,28 +70,29 @@ int main(int argc, char *argv[])
   {% endfor %} {# all synapse variables #}
   create_hidden_weightmatrix(brian::_dynamic_array_{{synapses.name}}__synaptic_pre, brian::_dynamic_array_{{synapses.name}}__synaptic_post, _hidden_weightmatrix{{synapses.name}},{{synapses.srcN}}, {{synapses.trgN}});
   {% else %} {# for sparse matrix representations #}
-  allocate{{synapses.name}}(brian::_dynamic_array_{{synapses.name}}__synaptic_pre.size());
   initialize_sparse_synapses(brian::_dynamic_array_{{synapses.name}}__synaptic_pre, brian::_dynamic_array_{{synapses.name}}__synaptic_post,
-                             C{{synapses.name}}, {{synapses.srcN}}, {{synapses.trgN}}, _{{synapses.name}}_bypre);
+                             rowLength{{synapses.name}}, ind{{synapses.name}}, maxRowLength{{synapses.name}},
+                             {{synapses.srcN}}, {{synapses.trgN}},
+                             sparseSynapseIndices{{synapses.name}});
   {% for var in synapses.variables %}
   {% if synapses.variablescope[var] == 'brian' %}
-  convert_dynamic_arrays_2_sparse_synapses(brian::_dynamic_array_{{synapses.name}}__synaptic_pre, brian::_dynamic_array_{{synapses.name}}__synaptic_post,
-                                           brian::_dynamic_array_{{synapses.name}}_{{var}}, {{var}}{{synapses.name}},
-                                           {{synapses.srcN}}, {{synapses.trgN}}, _{{synapses.name}}_bypre);
+  convert_dynamic_arrays_2_sparse_synapses(brian::_dynamic_array_{{synapses.name}}_{{var}},
+					   sparseSynapseIndices{{synapses.name}},
+                                           {{var}}{{synapses.name}},
+                                           {{synapses.srcN}}, {{synapses.trgN}});
   {% endif %}
   {% endfor %} {# all synapse variables #}
   {% endif %} {# dense/sparse #}
   {% for var in synapses.shared_variables %}
-  copy_brian_to_genn(brian::_array_{{synapses.name}}_{{var}}, &{{var}}{{synapses.name}}, 1);
+  std::copy_n(brian::_array_{{synapses.name}}_{{var}}, 1, &{{var}}{{synapses.name}});
   {% endfor %} {# shared variables #}
   {% endfor %} {# all synapse_models #}
-  initmagicnetwork_model();
 
   // copy variable arrays
   {% for neuron in neuron_models %} 
   {% for var in neuron.variables %}
   {% if neuron.variablescope[var] == 'brian' %}
-  copy_brian_to_genn(brian::_array_{{neuron.name}}_{{var}}, {{var}}{{neuron.name}}, {{neuron.N}});
+  std::copy_n(brian::_array_{{neuron.name}}_{{var}}, {{neuron.N}}, {{var}}{{neuron.name}});
   {% endif %}
   {% endfor %}
   {% endfor %}
@@ -103,33 +100,29 @@ int main(int argc, char *argv[])
   // copy scalar variables
   {% for neuron in neuron_models %}
   {% for var in neuron.shared_variables %}
-  copy_brian_to_genn(brian::_array_{{neuron.name}}_{{var}}, &{{var}}{{neuron.name}}, 1);
+  std::copy_n(brian::_array_{{neuron.name}}_{{var}}, 1, &{{var}}{{neuron.name}});
   {% endfor %}
   {% endfor %}
   
   // initialise random seeds (if any are used)
-  {% for neuron in neuron_models %} 
+  {% for neuron in neuron_models %} ;
   {% if '_seed' in neuron.variables %}
-  for (int i= 0; i < {{neuron.N}}; i++) {
-      _seed{{neuron.name}}[i]= (uint64_t) (rand()*MYRAND_MAX);
+  for (unsigned int i= 0; i < {{neuron.N}}; i++) {
+      _seed{{neuron.name}}[i]= (uint64_t) (rand()*0x0000FFFFFFFFFFFFLL);
   }
   {% endif %}
   {% endfor %}
   {% for synapses in synapse_models %}
   {% if '_seed' in synapses.variables %}
-  for (int i= 0; i < C{{synapses.name}}.connN; i++) {
-      _seed{{synapses.name}}[i]= (uint64_t) (rand()*MYRAND_MAX);
+  for (unsigned int i= 0; i < (maxRowLength{{synapses.name}} * {{synapses.srcN}}); i++) {
+      _seed{{synapses.name}}[i]= (uint64_t) (rand()*0x0000FFFFFFFFFFFFLL);
   }
   {% endif %}
   {% endfor %}
 
-  //-----------------------------------------------------------------
+  // Perform final stage of initialization, uploading manually initialized variables to GPU etc
+  initializeSparse();
   
-  eng.init(which);         // this includes copying g's for the GPU version
-#ifndef CPU_ONLY
-  copyStateToDevice();
-#endif
-
   //------------------------------------------------------------------
   // output general parameters to output file and start the simulation
   fprintf(stderr, "# We are running with fixed time step %f \n", DT);
@@ -137,26 +130,19 @@ int main(int argc, char *argv[])
   t= 0.;
   void *devPtr;
   {{'\n'.join(code_lines['before_run'])|autoindent}}
-  eng.run(totalTime, which); // run for the full duration
+  eng.run(totalTime); // run for the full duration
   {{'\n'.join(code_lines['after_run'])|autoindent}}
-  timer.stopTimer();
   cerr << t << " done ..." << endl;
   {% if prefs['devices.genn.kernel_timing'] %}
-  {% for kt in ('neuron_tme','synapse_tme','learning_tme','synDyn_tme') %}
-  {% if ktimer[kt] %}
+  {% for kt in ('neuronUpdateTime', 'presynapticUpdateTime', 'postsynapticUpdateTime', 'synapseDynamicsTime', 'initTime', 'initSparseTime') %}
   fprintf(timef,"%f ", {{kt}});
-  {% endif %}
   {% endfor %}
   {% endif %} 
-  fprintf(timef,"%f \n", timer.getElapsedTime());
 
   // get the final results from the GPU 
-#ifndef CPU_ONLY
-  if (which == GPU) {
-    eng.getStateFromGPU();
-    eng.getSpikesFromGPU();
-  }
-#endif
+  eng.getStateFromGPU();
+  eng.getSpikesFromGPU();
+
   // translate GeNN arrays back to synaptic arrays
   {% for synapses in synapse_models %}
   {% if synapses.connectivity == 'DENSE' %}
@@ -175,12 +161,13 @@ int main(int argc, char *argv[])
       {% else %}
       {% set _mode = 'b2g::COPY_ONLY' %}
       {% endif %}
-      convert_sparse_synapses_2_dynamic_arrays(C{{synapses.name}}, {{var}}{{synapses.name}}, {{synapses.srcN}}, {{synapses.trgN}}, brian::_dynamic_array_{{synapses.name}}__synaptic_pre, brian::_dynamic_array_{{synapses.name}}__synaptic_post, brian::_dynamic_array_{{synapses.name}}_{{var}}, {{_mode}});
+      convert_sparse_synapses_2_dynamic_arrays(rowLength{{synapses.name}}, ind{{synapses.name}}, maxRowLength{{synapses.name}},
+                                               {{var}}{{synapses.name}}, {{synapses.srcN}}, {{synapses.trgN}}, brian::_dynamic_array_{{synapses.name}}__synaptic_pre, brian::_dynamic_array_{{synapses.name}}__synaptic_post, brian::_dynamic_array_{{synapses.name}}_{{var}}, {{_mode}});
   {% endif %}
   {% endfor %} {# all synapse variables #}
   {% endif %} {# dense/sparse #}
   {% for var in synapses.shared_variables %}
-  copy_genn_to_brian(&{{var}}{{synapses.name}}, brian::_array_{{synapses.name}}_{{var}}, 1);
+  std::copy_n(&{{var}}{{synapses.name}}, 1, brian::_array_{{synapses.name}}_{{var}});
   {% endfor %} {# shared variables #}
   {% endfor %} {# all synapse_models #}
 
@@ -188,7 +175,7 @@ int main(int argc, char *argv[])
   {% for neuron in neuron_models %} 
   {% for var in neuron.variables %}
   {% if neuron.variablescope[var] == 'brian' %}
-  copy_genn_to_brian({{var}}{{neuron.name}}, brian::_array_{{neuron.name}}_{{var}}, {{neuron.N}});
+  std::copy_n({{var}}{{neuron.name}}, {{neuron.N}}, brian::_array_{{neuron.name}}_{{var}});
   {% endif %}
   {% endfor %}
   {% endfor %}
@@ -196,7 +183,7 @@ int main(int argc, char *argv[])
   // copy scalar variables
   {% for neuron in neuron_models %}
   {% for var in neuron.shared_variables %}
-  copy_genn_to_brian(&{{var}}{{neuron.name}}, brian::_array_{{neuron.name}}_{{var}}, 1);
+  std::copy_n(&{{var}}{{neuron.name}}, 1, brian::_array_{{neuron.name}}_{{var}});
   {% endfor %}
   {% endfor %}
 
@@ -220,38 +207,23 @@ int main(int argc, char *argv[])
 
 using namespace std;
 #include <cassert>
-#include "hr_time.h"
-
-#include "utils.h" // for CHECK_CUDA_ERRORS
-#include "stringUtils.h"
+#include <cstdint>
 #include <vector>
 
-#ifndef CPU_ONLY
-#include <cuda_runtime.h>
-#else
-#define __host__
-#define __device__
-#endif
-
-
-#ifndef RAND
-#define RAND(Y,X) Y = Y * 1103515245 +12345;X= (unsigned int)(Y >> 16) & 32767
-#endif
-
 // we will hard-code some stuff ... because at the end of the day that is 
 // what we will do for the CUDA version
 
 #define DBG_SIZE 10000
 
-// and some global variables
-CStopWatch timer;
 
 //----------------------------------------------------------------------
 // other stuff:
-// global variables for pre-calculated list of the postsynaptic targets ordered by presynaptic sources
+// These variables (if any) are needed to be able to efficiently copy brian
+// synapse variables into genn SPARSE synaptic arrays (needed for run_regularly)
+
 {% for synapses in synapse_models %}
 {% if synapses.connectivity != 'DENSE' %}
-vector<vector<int32_t> > _{{synapses.name}}_bypre;
+std::vector<size_t> sparseSynapseIndices{{synapses.name}};
 {% endif %}
 {% endfor %}
 {% endmacro %}
