@@ -2,8 +2,6 @@
 {% extends 'common_group.cpp' %}
 
 {% block extra_headers %}
-extern double t;
-#include "sparseProjection.h"
 #include "magicnetwork_model_CODE/definitions.h"
 {% endblock%}
 
